/************************************************************************
 *   MODULE  : VIS_MASK.C
 *
 *   PURPOSE : Fills the visibility product with the appropriate values.
 *
 *   AUTHOR  : Ron Hawkins & Craig Woodfolk, SAIC (original)
 *             Kevin Russo, SAIC (1, 3)
 *             Melissa Mathews, SAIC (2)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   25 Sep 92     n/a
 *   1          20 Oct 92     overall program update
 *   2          17 Jun 93     converted XVT 2.12 calls to XVT 3.01
 *   3          21 Jul 93     implemented new TEC LOS algorithm
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

/************************************************************************
 *   FUNCTION     : vis_mask
 *
 *   PURPOSE      : visibility mask building routine
 *
 *   INPUTS       : elev_hgts      - elevation buffer
 *                  prod_buff_ydim - product buffer (lat)
 *                  prod_buff_xdim - product buffer (lon)
 *                  profile_color  - profile color buffer
 *
 *   RETURN VALUE : none
 *
 *   CALLED FROM  : make_one_radial in module MAKE_RDL.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void vis_mask( short pHMEM elev_hgts, long pHMEM prod_buff_ydim,
               long pHMEM prod_buff_xdim, char *pHMEM profile_color )
#else
void vis_mask( elev_hgts, prod_buff_ydim, prod_buff_xdim, profile_color )
short  pHMEM elev_hgts;
long   pHMEM prod_buff_ydim;
long   pHMEM prod_buff_xdim;
char   *pHMEM profile_color;
#endif
{

   long    i, index;          /* counter                                */
   GFLOAT  distance;          /* distance to current point along radial */
   GFLOAT  vert_diff;         /* antenna top and target top difference  */
   GFLOAT  max_los_angle;     /* maximum LOS angle                      */
   GFLOAT  tgt_angle;         /* antenna to target angle                */
   GFLOAT  los_angle;         /* antenna to current LOS angle           */
   GFLOAT  correction;        /* elevation correction                   */
   GFLOAT  acuity_distance;   /* acuity factor                          */
   GFLOAT  eff_radius;        /* effective earth radius                 */

   char  pHMEM prfl_col;      /* profile color ptr                      */

   BYTE  vis, hid;

   extern PARAM_CALC      param;             /* calculations location   */
   extern SETTINGS_TYPE   settings;          /* user input parameters   */
   extern RASTER          *prod_raster;      /* product raster          */
   extern PREFS_TYPE      sys_prefs;         /* System Preferences      */
   extern char  pHMEM     prod1_ptr;         /* product data            */

   extern BYTE pHMEM      num_hidden;
   extern BYTE pHMEM      num_visible;

   /* set new local profile color pointer for more clarity */
   if (settings.profile)
      prfl_col = (char pHMEM)*profile_color;

   max_los_angle   = INIT_LOS_VAL;
   acuity_distance = settings.target.width / ACUITY_FACTOR;
   eff_radius      = (GFLOAT)2 * EARTH_RADIUS;

   for (i = ONE; i < param.nhts_in_radial; i++)
   {
      if (elev_hgts[i] < (short)ZERO)
         continue;

      index = prod_buff_ydim[i] * prod_raster->data_matrix.width +
              prod_buff_xdim[i];

      distance    = i * param.radial_step_size;
      correction  = (GFLOAT)elev_hgts[i] - ((distance * distance) / eff_radius);
      vert_diff   = correction - (GFLOAT)elev_hgts[0] -
                   (GFLOAT) settings.observer.height +
                   (GFLOAT) settings.target.height;

      tgt_angle   = atan(vert_diff / distance) * ARCTAN_DEGREES;

      los_angle   = atan((vert_diff - settings.target.height) /
                    distance) * ARCTAN_DEGREES;

      if (los_angle > max_los_angle)
         max_los_angle = los_angle;

      if ((tgt_angle < max_los_angle) ||
          ((sys_prefs.acuity == TRUE) && (distance > acuity_distance)))
      {
         if ((sys_prefs.acuity == TRUE) && (distance > acuity_distance))
         {
            prod1_ptr[index] = (char)BEYOND_ACUITY;
         }
         else
         {
            /* visibility priority mask */
            num_hidden[index]++;
         }
      }
      else
      {
         /* tgt_angle > max_los_angle */
         num_visible[index]++;
      }
   } /* end of param.nhts_in_radial loop */


   /* assign color codes to profile array */
   if (settings.profile)
   {
      for (i = ONE; i < param.nhts_in_radial; i++)
      {
         index = prod_buff_ydim[i] * prod_raster->data_matrix.width +
                 prod_buff_xdim[i];

         if (elev_hgts[i] < (short)ZERO)
         {
            prfl_col[i] = (char)ZERO;
         }
         else if (prod1_ptr[index] == (char)BEYOND_ACUITY)
         {
            prfl_col[i] = (char)BEYOND_ACUITY;
         }
         else
         {
            hid = num_hidden[index];

            vis = num_visible[index];

            if ((hid == (BYTE)ZERO) && (vis == (BYTE)ZERO))
               continue;

            if (prod1_ptr[index] != (char)ZERO)
               continue;

            if (vis >= hid)
            {
               prod1_ptr[index] = (char)DEF_VISIBLE;

               prfl_col[i] = (char)DEF_VISIBLE;
            }
            else
            {
               prod1_ptr[index] = (char)DEF_HIDDEN;
   
               prfl_col[i] = (char)DEF_HIDDEN;
            }
         }
      }
   }
}
