/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */


/*
 * MapPrimitive -- a geo-located element with topological structure
 */

#ifndef mapprimitive_h
#define mapprimitive_h

#include <Unidraw/iterator.h>

class MapFeature;
class TopoEdge;
class TopoEdgeList;
class TopoElement;
class TopoFace;
class TopoNode;

class MapPrimitive {
public:
    MapPrimitive(MapFeature* = nil);
    virtual ~MapPrimitive();

    void feature(MapFeature*);
    MapFeature* feature() const;

    virtual int npts() const;

    virtual const float* xpoints() const;
    virtual const float* ypoints() const;
    virtual const float* zpoints() const;

    virtual void origin(float& x, float &y, float &z) const;

    virtual float xorigin() const;
    virtual float yorigin() const;
    virtual float zorigin() const;

    virtual float xextent() const;
    virtual float yextent() const;
    virtual float zextent() const;

    TopoElement* topo() const { return _topo; }

    /* topology dependent methods */
    virtual void insert_points(int npts, float *x, float *y, float *z=nil);
    virtual void insert_points(int npts, int *x, int *y, int *z=nil);
    virtual void insert_pointers(int npts, float *x, float *y, float *z=nil, boolean freeflag = false);

protected:
    MapFeature* _feature;
    TopoElement* _topo;
};

class MapFace;
class MapNode;

class MapEdge : public MapPrimitive {
public:
    MapEdge(MapFeature* = nil, boolean topoflag = true);

    virtual MapFace* left_face() const;
    virtual MapFace* right_face() const;

    virtual MapNode* start_node() const;
    virtual MapNode* end_node() const;

    virtual MapEdge* next_edge(MapFace*, MapNode*) const;
    virtual boolean forward(MapNode*) const;

    TopoEdge* topo() const { return (TopoEdge*)_topo; }

    /* topology dependent methods */
    virtual void attach_nodes(MapNode* start, MapNode* end);
    virtual void attach_faces(MapFace* left, MapFace* right);
};

class MapEdgeList : public MapPrimitive {
public:
    MapEdgeList(MapFeature* = nil);

    // Iterator methods for walking adjacent edges
    virtual void first(Iterator&) const; 
    virtual void last(Iterator&) const;
    virtual void next(Iterator&) const;
    virtual void prev(Iterator&) const;
    virtual boolean done(Iterator) const;
    virtual boolean is_empty() const;
    virtual int number() const;

    TopoEdgeList* topo() const { return (TopoEdgeList*)_topo; }

    /* topology dependent methods */
    void append(MapEdge*);
    void prepend(MapEdge*);
};

class MapFace : public MapEdgeList {
public:
    MapFace(MapFeature* = nil, boolean topoflag = true);

    virtual int npts() const;
    virtual boolean clockwise(MapEdge*) const;

    TopoFace* topo() const { return (TopoFace*)_topo; }

};

class MapNode : public MapEdgeList {
public:
    MapNode(MapFeature* = nil, boolean topoflag = true);

    TopoNode* topo() const { return (TopoNode*)_topo; }


};

class MapPoint : public MapEdge {
public:
    MapPoint(MapFeature* = nil, boolean topoflag = true);

    virtual MapFace* containing_face() const;
};

class MapText : public MapPoint {
public:
    MapText(MapFeature* = nil, boolean topoflag = true);
    virtual ~MapText();

    virtual const char* text() const;
    virtual void text(const char*);
protected:
    char* _text;
};

class MapRaster : public MapFace {
public:
    MapRaster(MapFeature* = nil, boolean topoflag = true);

};

class MapDem : public MapFace {
public:
    MapDem(MapFeature* = nil, boolean topoflag = true);

};

#include <InterViews/_leave.h>


#endif
